function [a, action] = asfq_greedyact(a, state, actions, rewards, params)
%Implements deterministic greedy action selection for adaptive state focus Q-learning
%  [A, ACTION] = ASFQ_GREEDYACT(A, STATE, ACTIONS, REWARDS, PARAMS)
%  Implements deterministic greedy action selection, to be combined with
%  the adaptive state focus Q-learning algorithm. The action with the
%  highest Q-value is chosen (breaking ties randomly). The action selection
%  mechanism takes into account the learning mode of the agent
%  (single-agent, or full state), and thus the structure of the Q-table.
%
%  Supports discrete states and actions, with 1 action variable only.
%
%  Requires the size of the Q-table to be cached under a field named
%  'qsize'.
%
%
%  See also agent_act, asfq

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


if isfield(params, 'mode') && strcmp(params.mode, 'init'),
    return;         % no init operations for this type of policy
end;

switch a.learnmode,
    case 1,
        ix = ndi2lin([1 state(a.indices(1) : a.indices(2))'], a.qsize);
    case 2,
        ix = ndi2lin([1 state'], a.qsize);
end;
ix = ix : ix + a.actionscount - 1;
topactions = find(a.Q(ix) == max(a.Q(ix)));
action = topactions(ceil(rand * length(topactions)));

% returning a, action
% END greedyact() =====================================================