function [a, action] = greedyact(a, state, actions, rewards, params)
%Implements deterministic greedy action selection for Q-learners
%  [A, ACTION] = GREEDYACT(A, STATE, ACTIONS, REWARDS, PARAMS)
%  Implements greedy deterministic action selection, using a Q-table
%  indexed by the agent's state and action. The action with the highest
%  Q-value is chosen (breaking ties randomly).
%
%  Supports discrete states and actions, with 1 action variable only.
%
%  Requires the size of the Q-table to be cached under a field named
%  'qsize'.
%
%  See also act

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:

if isfield(params, 'mode') && strcmp(params.mode, 'init'),
    return;         % no init operations for this type of policy
end;

ix = ndi2lin([1 state(a.indices(1) : a.indices(2))'], a.qsize);
ix = ix : ix + a.actionscount - 1;
topactions = find(a.Q(ix) == max(a.Q(ix)));
action = topactions(ceil(rand * length(topactions)));

% returning a, action
% END greedyact() =====================================================