function [a, action] = stochact(a, state, actions, rewards, params)
%Implements stochastic action selection for policy indexed on agent's state
%  [A, ACTION] = STOCHACT(A, STATE, ACTIONS, REWARDS, PARAMS)
%  Implements stochastic action selection. An action is chosen based on the
%  current stochastic policy of the agent. The elements of the policy
%  corresponding to a given state must form a valid probability
%  distribution over the (discrete) actions.
%
%  Supports discrete states and actions, with 1 action variable only.
%
%  This policy must be stored under field 'PI' of the agent, as a flat
%  vector representing a matrix with dimensions agent-action-space-size X
%  agent-state-space-size. Moreover, this size must be cached in field
%  'sizes.pi' of the agent.
%
%
%  See also agent_act

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:

if isfield(params, 'mode') && strcmp(params.mode, 'init'),
    DEF_LP.usefullstate = 1;
    a.learnparam = checkparams(a.learnparam, DEF_LP);
    if a.learnparam.usefullstate,
        a.xindices = 1:size(params.info.statespacesize, 1);
    else
        a.xindices = a.indices(1):a.indices(2);
    end;
    return;         % no init operations for this type of policy
end;

piix = ndi2lin([1 a.state(a.xindices)'], a.sizes.pi);
policy = a.PI(piix : piix + a.actionscount - 1);
action = find(rand <= cumsum(policy), 1);

% returning a, action
% END stochact() ==================================================