function a = distrq(a, state, action, reward, params)
%Implements the distributed Q-learning algorithm
%  A = TEAMQ(A, STATE, ACTION, REWARD, PARAMS)
%  Implements distributed Q-learning. 
%
%  Required values on the agent learning parameters:
%   gamma           - the discount factor
%   epsilon         - the exploration probability
% 
%  Required values on the extra parameters:
%   newtrial        - only in episodic environments; whether a new trial is
%                   beginning
%
% Supports discrete states and actions, with 1 action variable per agent.
%
% Should be coupled with determpiact.
%
%
%  References:
%  [1] Lauer, M. & Riedmiller, M. An Algorithm for Distributed
%  Reinforcement Learning in Cooperative Multi-Agent Systems Proceedings
%  17th International Conference on Machine Learning (ICML-00), 2000,
%  535-542
%
%  See also agent_learn, distrq_init

%  Author:      Lucian Busoniu
%  Version:     1

% convert both old index; start index of new state; start index of old state at once
qix = ndi2lin([action(a.indices(5)) a.state'; 1 state'; 1 a.state'], a.sizes.q);
Qxold = a.Q(qix(3) : qix(3) + a.actionscount - 1);
% "optimistic" update of distributed Q
a.Q(qix(1)) = max(a.Q(qix(1)), reward(a.indices(7)) + a.learnparam.gamma * max(a.Q(qix(2) : qix(2) + a.actionscount - 1)));

if a.Q(qix(1)) > max(Qxold),
    a.h(ndi2lin(a.state', a.sizes.h)) = action(a.indices(5));
end;

% returning a
% END teamq() ============================================================
