function a = distrq_init(a, info)
%Initializes the distributed Q-learning algorithm
%  A = TEAMQ_INIT(A, INFO)
%  Values on the info parameters:
%   statespacesize      - the state space size
%   actionspacesizes    - the sizes of the action spaces for all agents, vector
%       containing the number of actions available to each agent
% 
%  See also distrq, agent_initlearn

%  Author:      Lucian Busoniu
%  Version:     1.0

% Check states and actions
if any(info.statespacesize(:, 1) ~= 1) || ...
        a.indices(5) < a.indices(6) || ...
        any(info.actionspacesize(:, 1) ~= 1)
    error('teamq() only supports discrete states and actions, with 1 action variable');
else
    a.actionscount = info.actionspacesize(info.imap(5), 2);
end;

% init structures
a.sizes.q = [a.actionscount info.statespacesize(:, 2)'];
a.sizes.h = info.statespacesize(:, 2)';
a.Q = zeros(prod(a.sizes.q), 1);
a.h = unidrnd(a.actionscount, prod(a.sizes.h), 1);     % randomly initialized policy for better exploration

% returning a
% END teamq_init()
% ========================================================