function a = fullstate_plainq(a, state, action, reward, params)
%Implements the full-state plain Q-learning algorithm
%  A = FULLSTATE_PLAINQ(A, STATE, ACTION, REWARD, PARAMS)
%  Implements full-state version of the plain Q-learning algorithm,
%  employing in addition to that an eligibility trace. Uses flat Q- and
%  eligibility tables for fast access.
%
%  Supports discrete states and actions, with 1 action variable per agent.
%
%  Required values on the agent learning parameters:
%   alpha           - the learning rate
%   gamma           - the discount factor
%   lambda          - the eligibility trace decay rate
%   epsilon         - the exploration probability
%  Required values on the extra parameters:
%   newtrial        - only in episodic environments; whether a new trial is
%                   beginning
%
% Can be coupled with an action function that uses a Q-table indexed on
% full world state and agent action, such as fullstate_greedyact().
%
%
%  See also agent_learn, plainq, fullstate_plainq_init

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


if a.learnparam.episodic && params.newtrial, 
    a.E = 0 * a.Q;          % clear eligibility trace
end;

ix = ndi2lin([1 state'], a.qsize);
qix = ndi2lin([action(a.indices(5)) a.state'], a.qsize);

delta = reward(a.indices(7)) + a.learnparam.gamma * max(a.Q(ix : ix + a.actionscount - 1)) - a.Q(qix);
a.E(qix) = 1;
a.Q = a.Q + a.learnparam.alpha * delta * a.E;
a.E = a.learnparam.gamma * a.learnparam.lambda * a.E * ~a.exploring;

% returning a
% END plainq() ============================================================
