function a = teamq_init(a, info)
%Initializes the team Q-learning algorithm
%  A = TEAMQ_INIT(A, INFO)
%  Creates the structures required for the team Q-learning algorithm to
%  run.
%  Values on the info parameters:
%   statespacesize      - the state space size
%   actionspacesizes    - the sizes of the action spaces for all agents, vector
%       containing the number of actions available to each agent
% 
%
%  See also teamq, agent_initlearn

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:
%   - 12 Jan 2006, changed to use full state

% Check states and actions
if any(info.statespacesize(:, 1) ~= 1) || ...
        a.indices(5) < a.indices(6) || ...
        any(info.actionspacesize(:, 1) ~= 1)
    error('teamq() only supports discrete states and actions, with 1 action variable');
else
    a.actionscount = info.actionspacesize(info.imap(5), 2);
end;

% install default params
DEF_LP.alphadecay = 1;
DEF_LP.lambda = 0;
a.learnparam = checkparams(a.learnparam, DEF_LP);
a.useelig = (a.learnparam.lambda > 0);      % if not using elig traces, enables optimization in learning algorithm

% init structures
a.sizes.act = info.actionspacesize(:, 2)';
a.sizes.expact = prod(a.sizes.act);
a.sizes.q = [a.sizes.act info.statespacesize(:, 2)'];
a.Q = zeros(prod(a.sizes.q), 1);

if a.useelig,
    a.E = a.Q;
    % E is volatile
    a.volatile = {'E'};
% else disp('Running optimized version');
end;

% returning a
% END teamq_init()
% ========================================================