function [world, agents, stats] = learn(world, agents, learnparam)
%Implements the learning control mechanism
%  [WORLD, AGENTS, STATS] = LEARN(WORLD, AGENTS, LEARNPARAM)
%
%  Runs the learning algorithms of the agents in the world. Delegates the
%  control to either episodic_learn() or continuing_learn(), depending on
%  the task type offerred by the world.
%
%  Note that currently, only episodic_learn() is implemented.
%
%  Parameters:
%   WORLD       - the world within which the learning takes place.
%   AGENTS      - the (possibly heterogeneous) cell array of agents.
%               Must be the same with which the world was created.
%   LEARNPARAM  - the learning parameters
%
%  Returns:
%   WORLD       - the possibly altered world.
%   AGENTS      - the (savvier) agents.
%   STATS       - the learning statistics.
%
%
%  See also episodic_learn

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


REQ_ARG_COUNT = 3;
if nargin < REQ_ARG_COUNT, error('Incorrect number of inputs'); end;

switch world.tasktype,
    case 'episodic',
        [world, agents, stats] = episodic_learn(world, agents, learnparam);
%     case 'continuing',      % NOTE: continuing tasks mechanism not yet implemented
%         [world, agents, stats] = continuing_learn(world, agents, learnparam);
    otherwise,
        error(['Invalid task type ' world.tasktype]);
end;

% returning world, agents, stats

% END learn() =============================================================