function [world, agents, stats] = replay(world, agents, speed, maxiter)
%Replays the agents' current policies
%  [WORLD, AGENTS[, STATS]] = REPLAY(WORLD, AGENTS[, SPEED[, MAXITER]])
%  Replays the agents' policies over a single trial. Assumes that the
%  agents have already been correctly setup during the learning process.
%
%  Parameters:
%   WORLD       - the world where the agents live
%   AGENTS      - the (possibly heterogeneous) cell array of agents
%   SPEED       - (optional) the speed of the replay. Ranges between 1 and
%               10, 1 is (roughly) one second per iteration, 10 is full
%               speed. Default is 7.
%   MAXITER    - (optional) how much to allow the policy to run at most.
%               Defaults to 1000 iterations. May be -1 ('run forever').
%  Returns:
%   WORLD       - the world, altered by the agents' actions during replay
%   AGENTS      - the possibly altered agents.
%   STATS       - replay statistics such as:
%       'iter'      - how many iterations it took to terminate the task
%       'stuck'     - boolean flag signaling whether the agents got stuck
%
%
%  See also learn

%  Author:      Lucian Busoniu
%  Version:     1.0
%  History:


DEFAULT_SPEED = 7; DEFAULT_MAXSTEPS = 1000;
REQ_ARG_COUNT = 2;
if nargin < REQ_ARG_COUNT, error('replay() requires at least 2 inputs'); end;
if nargin < REQ_ARG_COUNT + 1, speed = DEFAULT_SPEED; end;
if nargin < REQ_ARG_COUNT + 2, maxiter = DEFAULT_MAXSTEPS; end;

switch world.tasktype,
    case 'episodic',
        [world agents stats] = episodic_replay(world, agents, speed, maxiter);
%     case 'continuing',
%         [world agents stats] = continuing_replay(world, agents, speed, maxiter);
    otherwise,
        error(['Invalid task type ' world.tasktype]);
end;

% returning world, agents, stats

% END replay() ============================================================