function varargout = alg_gui(varargin)
% ALG_GUI MARL algorithms demonstration GUI.
% 
% Use ALG_GUI to start the demonstration interface.
%
% This demonstration shows the performance of various MARL algorithms
% on two types of tasks - fully cooperative and mixed. Select a type of
% task from the left listbox, an algorithm from the right, and click "Run"
% to run learning. Once learning has been run, the learned policies can be
% replayed by clicking "Replay", and the convergence of the algorithm can
% be verified by clicking "Conv. Plot". The "[ Optimal ]" items from the
% right listbox do not run learning, but instead load a predefined set of
% optimal policies on the given task.
%
% Learning and replaying can be customized from the configuration panel.
% The replay speed (from 0 - step by step - to 10 - full CPU speed) can be
% set via the sliding rule "Replay speed".
% 
% The progress of learning can be shown "live" during the entire process,
% or starting from a given trial. For the latter, tick the "Show only after
% #trials" checkbox on, and move the sliding rule to set this number of
% trials. The progress of learning can be slowed down for visualization
% after a given trial. To do this, tick the "Slow down after #trials"
% checkbox on, and move the sliding rule to set this number of trials.
%
% Tick "Pause before run" to have the chance to examine a new world prior
% to the learning process being initiated. Tick "Use Star Wars robots" to
% use images of Star Wars robots to represent robots, instead of abstract
% colored discs. The goals then transform into color-coded power sockets.
%
% Quit the demo by pressing "Quit" or the Close button on the window.
%   
%
% See also: run_algorithm, replay_algorithm

% Edit the above text to modify the response to help alg_gui

% Last Modified by GUIDE v2.5 08-Nov-2006 17:18:43

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @alg_gui_OpeningFcn, ...
                   'gui_OutputFcn',  @alg_gui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before alg_gui is made visible.
function alg_gui_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to alg_gui (see VARARGIN)

% Choose default command line output for alg_gui
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes alg_gui wait for user response (see UIRESUME)
% uiwait(handles.alg_gui);

% load the RL toolbox
% startuprl

% --- Outputs from this function are returned to the command line.
function varargout = alg_gui_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on selection change in TaskListBox.
function TaskListBox_Callback(hObject, eventdata, handles)

% END TaskListBox_Callback() ----------------------------------------

% --- Executes during object creation, after setting all properties.
function TaskListBox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in CoopListBox.
function CoopListBox_Callback(hObject, eventdata, handles)

% --- Executes during object creation, after setting all properties.
function CoopListBox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in ReplayButton.
function ReplayButton_Callback(hObject, eventdata, handles)
internal_replay_algorithm(handles);
% END ReplayButton_Callback() --------------------------------------------


% --- Executes on button press in ConvPlotButton.
function ConvPlotButton_Callback(hObject, eventdata, handles)
[task, cooplev, method] = PullSelection(handles);
if strcmp(method, 'opt'), warning('No convergence data for (precomputed) optimal policies.'); return; end;

try
    datvar = ['algdata.' task '_' cooplev '_' method];
    dat = evalin('base', datvar);
    stats = dat.stats;

    figh = figure;
    set(figh, 'Position', [0 0 700 500]); movegui(figh, 'center');
    stairs(stats.iter, 'k', 'LineWidth', 2); grid;
    xlabel('Learning trial'); ylabel('Steps to complete trial');
    title(['Learning convergence. Total learning time: ' num2str(stats.elapsed_time) ' sec.']);
catch
    warning(['No convergence data for ' task '/' cooplev '/' method]);
end;
% END ReplayButton_Callback() --------------------------------------------


% --- Executes on button press in RunButton.
function RunButton_Callback(hObject, eventdata, handles)
internal_run_algorithm(handles);
% END RunButton_Callback() --------------------------------------------

% --- Executes on selection change in MixedListBox.
function MixedListBox_Callback(hObject, eventdata, handles)


% --- Executes during object creation, after setting all properties.
function MixedListBox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in CompListBox.
function CompListBox_Callback(hObject, eventdata, handles)

% --- Executes during object creation, after setting all properties.
function CompListBox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in showafter_toggle.
function showafter_toggle_Callback(hObject, eventdata, handles)


% --- Executes on button press in slowdownafter_toggle.
function slowdownafter_toggle_Callback(hObject, eventdata, handles)

% --- Executes on button press in userobots_toggle.
function userobots_toggle_Callback(hObject, eventdata, handles)

% --- Executes on slider movement.
function replayspeedvalue_Callback(hObject, eventdata, handles)
set(hObject, 'Value', round(get(hObject, 'Value')));
set(handles.replayspeedvalue_feedback, 'String', ...
    num2str(get(hObject, 'Value')));
% END -----------------------------------------------


% --- Executes during object creation, after setting all properties.
function replayspeedvalue_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

function replayspeedvalue_feedback_Callback(hObject, eventdata, handles)

% --- Executes during object creation, after setting all properties.
function replayspeedvalue_feedback_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on slider movement.
function showaftervalue_Callback(hObject, eventdata, handles)
set(hObject, 'Value', round(get(hObject, 'Value')));
set(handles.showaftervalue_feedback, 'String', ...
    num2str(get(hObject, 'Value')));


% --- Executes during object creation, after setting all properties.
function showaftervalue_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

function showaftervalue_feedback_Callback(hObject, eventdata, handles)

% --- Executes during object creation, after setting all properties.
function showaftervalue_feedback_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on slider movement.
function slowdownaftervalue_Callback(hObject, eventdata, handles)
set(hObject, 'Value', round(get(hObject, 'Value')));
set(handles.slowdownaftervalue_feedback, 'String', ...
    num2str(get(hObject, 'Value')));


% --- Executes during object creation, after setting all properties.
function slowdownaftervalue_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

function slowdownaftervalue_feedback_Callback(hObject, eventdata, handles)

% --- Executes during object creation, after setting all properties.
function slowdownaftervalue_feedback_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pausebeforerun_toggle.
function pausebeforerun_toggle_Callback(hObject, eventdata, handles)


% --- Executes on button press in QuitButton.
function QuitButton_Callback(hObject, eventdata, handles)
% delete(gcf);
close(handles.alg_gui);

% END ------------------------------


% --- Executes on selection change in CoopLevelListBox.
function CoopLevelListBox_Callback(hObject, eventdata, handles)
sync_algorithm_listbox(handles);

% --- Executes during object creation, after setting all properties.
function CoopLevelListBox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in AutoRunButton.
% Autoruns algorithms. Interrupted by any error. Typically this will be 
% a close of the world GUI.
function AutoRunButton_Callback(hObject, eventdata, handles)
disp('Autorunning all algorithms for current task.');
pbr = get(handles.pausebeforerun_toggle, 'Value');
try
    % no pause before run in autorun
    set(handles.pausebeforerun_toggle, 'Value', 0);
    % task is fixed, we only loop through cooperation levels and algorithms
    ncoop = length(get(handles.CoopLevelListBox, 'String'));
    % for each cooperation level...
    for ic = 1:ncoop,
        % visual feedback
        set(handles.CoopLevelListBox, 'Value', ic);
        sync_algorithm_listbox(handles);
        switch ListBoxIndexToLabel('CoopLevelListBox', ic)
            case 'coop', halg = handles.CoopListBox;
            case 'mixed', halg = handles.MixedListBox;
        end;
        nalg = length(get(halg, 'String'));
        % ...run each method
        for ia = 1:nalg,
            set(halg, 'Value', ia);
            internal_run_algorithm(handles);
        end;
    end;
catch       % assume the GUI has been closed
    % make sure there's no data referring to the world,
    % it would be inconsistent
    try
        % try removing convergence plot
        if ishandle(899), delete(899); end;
        % remove algorithm data
        [task, cooplev, method] = PullSelection(handles);
        datvar = [task '_' cooplev '_' method];
        algdata = evalin('base', 'algdata');
        algdata = rmfield(algdata, datvar);
        assignin('base', 'algdata', algdata);
    catch
        % might happen if we closed before data was saved
    end;    
    disp('Autorun was interrupted.');
end;
% restore setting for pause before run
set(handles.pausebeforerun_toggle, 'Value', pbr);
% END AutoRunButton_Callback -------------------------------------------


% ===============================================
% === HERE START INTERNAL UTILITY FUNCTIONS =====
% ===============================================


% =======================================================================
% Internal function for running algorithms. Called from several places,
% mainly Run button callback
function internal_run_algorithm(handles)
[task, cooplev, method] = PullSelection(handles);
run_algorithm(task, cooplev, method, PullConfig(handles));
% END function ==========================================================

% =======================================================================
% Internal function for replaying algorithms. Called from several places,
% mainly Replay button callback
function internal_replay_algorithm(handles)
[task, cooplev, method] = PullSelection(handles);
replay_algorithm(task, cooplev, method, PullConfig(handles));
% END function ==========================================================

% =======================================================================
% Show the algorithm listbox given by the cooperation level
function sync_algorithm_listbox(handles)
switch ListBoxIndexToLabel('CoopLevelListBox', get(handles.CoopLevelListBox,'Value')),
    case 'coop',
        set(handles.CoopListBox, 'Visible', 'on');
        set(handles.MixedListBox, 'Visible', 'off');
    case 'mixed',
        set(handles.CoopListBox, 'Visible', 'off');
        set(handles.MixedListBox, 'Visible', 'on');
end;
% END function ==========================================================

% =======================================================================
% Pull config from GUI elements
function cfg = PullConfig(handles)

cfg.replayspeed = get(handles.replayspeedvalue, 'Value');

if get(handles.showafter_toggle, 'Value'),
    vcfg.showafter = round(get(handles.showaftervalue, 'Value'));
else vcfg.showafter = Inf;
end;

if get(handles.slowdownafter_toggle, 'Value'),
    vcfg.slowdownafter = round(get(handles.slowdownaftervalue, 'Value'));
else vcfg.slowdownafter = Inf;
end;

% these are the view configs
cfg.viewcfg = vcfg;
cfg.pausebeforerun = get(handles.pausebeforerun_toggle, 'Value');
% END function ==========================================================


% =======================================================================
% Pull selection of task, cooperation level and method from list boxes
function [task, cooplev, method] = PullSelection(handles)
task = ListBoxIndexToLabel('TaskListBox', get(handles.TaskListBox,'Value'));
cooplev = ListBoxIndexToLabel('CoopLevelListBox', get(handles.CoopLevelListBox,'Value'));
switch cooplev,
    case 'coop',
        method = ListBoxIndexToLabel('CoopListBox', get(handles.CoopListBox,'Value'));
    case 'mixed',
        method = ListBoxIndexToLabel('MixedListBox', get(handles.MixedListBox,'Value'));
end;
% END function ==========================================================


% =======================================================================
% Convert numeric list box index to string label, for all listboxes
% in the GUI
function label = ListBoxIndexToLabel(tag, index)

TASKS = {'rescue', 'nav'};
COOPLEVELS = {'coop', 'mixed'};
COOP_MET = {'q', 'fsq', 'asf', 'wolf', 'teamq', 'opt'};
MIXED_MET = {'q', 'fsq', 'asf', 'wolf', 'singleq', 'opt'};

switch tag,
    case 'TaskListBox',
        label  = TASKS{index};        
    case 'CoopLevelListBox',
        label  = COOPLEVELS{index};        
    case 'CoopListBox',
        label  = COOP_MET{index};
    case 'MixedListBox',
        label  = MIXED_MET{index};
    otherwise
        warning('Unknown tag');
end;    
% END function ==========================================================
